using BPA.SAAS.Manage.Application.DataBase.Dtos.GoodsAttribute;
using BPA.SAAS.Manage.Core.Base;
using BPA.SAAS.Manage.Core.DataBase;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace BPA.SAAS.Manage.Application.DataBase.Interface
{
    public interface IGoodsAttributeService
    {
        /// <summary>
        /// 分页查询
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        PageUtil GetGoodsAttributePageAsync(GoodsAttributeQueryDto dto);
        /// <summary>
        /// 查询商品属性（外部调用）
        /// </summary>
        /// <returns></returns>
        Task<List<GoodsAttributeView>> GetGoodsAttributeList_alm();
        /// <summary>
        /// 添加/修改
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        Task<bool> AddGoodsAttribute(GoodsAttributeDto dto);
        /// <summary>
        /// 删除
        /// </summary>
        /// <param name="Ids"></param>
        /// <returns></returns>
        Task<bool> DeleteGoodsAttribute(string[] Ids);
        /// <summary>
        /// 查询商品属性值列表
        /// </summary>
        /// <param name="goodsAttributeId"></param>
        /// <returns></returns>
        Task<List<BPA_GoodsAttributeValue>> GetGoodsAttributeValueAsync(string goodsAttributeId);
        /// <summary>
        /// 添加/修改
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        Task<bool> AddGoodsAttributeValue(List<GoodsAttributeValueDto> dto);
        /// <summary>
        /// 删除
        /// </summary>
        /// <param name="id"></param>
        /// <returns></returns>
        Task<bool> DeleteGoodsAttributeValue(string id);
        /// <summary>
        /// 根据商品id查询商品属性
        /// </summary>
        /// <param name="id"></param>
        /// <returns></returns>
        Task<List<GoodsAttributeList>> GetByGoodsIdAttribute(string id);
        Task<List<GoodsAttributeList>> GetByNameAttribute(string name);
    }
}
